# name: systemd time span
# desc: function for systemd time spans
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:


function systemd::time_span (
	Struct[{
			doupental_weeks               => Optional[Integer],
			quad_weeks                    => Optional[Integer],
			weeks                         => Optional[Integer],
			days                          => Optional[Integer],
			hours                         => Optional[Integer],
			minutes                       => Optional[Integer],
			seconds                       => Optional[Integer],
		}] $duration = {
			doupental_weeks               => 0,
			quad_weeks                    => 0,
			weeks                         => 0,
			days                          => 0,
			hours                         => 0,
			minutes                       => 0,
			seconds                       => 0,
		},
	Boolean                  $restrict_normalized           = true,
	Boolean                  $restrict_non_negative_result  = true,
) {
	# I N P U T   M A P P I N G
	$_doupental_weeks              = $duration['doupental_weeks'] ? {
		undef                         => 0,
		default                       => $duration['doupental_weeks'],
	}
	
	$_quad_weeks                   = $duration['quad_weeks'] ? {
		undef                         => 0,
		default                       => $duration['quad_weeks'],
	}
	
	$_weeks                        = $duration['weeks'] ? {
		undef                         => 0,
		default                       => $duration['weeks'],
	}
	
	$_days                         = $duration['days'] ? {
		undef                         => 0,
		default                       => $duration['days'],
	}
	
	$_hours                        = $duration['hours'] ? {
		undef                         => 0,
		default                       => $duration['hours'],
	}
	
	$_minutes                      = $duration['minutes'] ? {
		undef                         => 0,
		default                       => $duration['minutes'],
	}
	
	$_seconds                      = $duration['seconds'] ? {
		undef                         => 0,
		default                       => $duration['seconds'],
	}
	
	# I N P U T   V A L I D A T I O N
	if $restrict_input_values {
		if ($_quad_weeks < 0) or ($_quad_weeks > 11) {
			fail('quad_weeks must be within [0, 11]')
		}
		if ($_weeks < 0) or ($_weeks > 51) {
			fail('weeks must be within [0, 51]')
		}
		if ($_days < 0) or ($_days > 6) {
			fail('days must be within [0, 6]')
		}
		if ($_hours < 0) or ($_hours > 23) {
			fail('hours must be within [0, 23]')
		}
		if ($_minutes < 0) or ($_minutes > 59) {
			fail('minutes must be within [0, 59]')
		}
		if ($_seconds < 0) or ($_seconds > 59) {
			fail('seconds must be within [0, 59]')
		}
		# doupental_weeks is this function's largest accepted unit
		# do not restrict its upper bound
		if ($_doupental_weeks < 0) {
			fail('doupental_weeks must be within [0, infty)')
		}
	}
	
	# C A L C U L A T I O N
	$_result = $_seconds +
		60 * $_minutes +
		60 * 60 * $_hours +
		60 * 60 * 24 * $_days +
		60 * 60 * 24 * 7 * $_weeks +
		60 * 60 * 24 * 7 * 4 * $_quad_weeks +
		60 * 60 * 24 * 7 * 4 * 12 * $_doupental_weeks
	
	# O U T P U T   V A L I D A T I O N
	if ($restrict_non_negative_result) and ($result < 0) {
		fail('non-negative restriction on: got negative result')
	}
	
	# R E T U R N
	$_result
}
# EOF
