# name: configure systemd
# desc: configuration manifest for systemd module
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker:

# naming convention:
# systemd specific resources were named with a 'systemd_' prefix
# whereas annexed resources followed the normal ruby identifier naming scheme
# e.g. 'locale_conf', 'udev_conf' or 'sysctl_conf'

# user documentation as YARD documentation {{{
# This is a private class.
# 
# * It generates and places configuration files.
# * Or invokes configure commands.
# * Optionally it collects several resources.
# 
# }}}
class systemd::config {
	# V A R I A B L E   I N I T
	
	# also used in systemd::daemon
	# lint:ignore:only_variable_string
	$_pending_removal = "${systemd::systemd_package_version}" =~ /^(absent|purged)$/
	# lint:endignore
	
	
	# D E F I N E D   T Y P E S   I N T E G R A T I O N
	
	# avoid using cached unit files:
	
	if $systemd::collect_systemd_resources and ! $_pending_removal {
		Systemd::Automount <| |> ~> Exec['systemd_reload_configuration']
		Systemd::Device <| |>    ~> Exec['systemd_reload_configuration']
		Systemd::Mount <| |>     ~> Exec['systemd_reload_configuration']
		Systemd::Path <| |>      ~> Exec['systemd_reload_configuration']
		Systemd::Scope <| |>     ~> Exec['systemd_reload_configuration']
		# only those systemd::service, which do not manage the service resource
		# (otherwise dependency cycle!)
		Systemd::Service <| manage_service_resource != true |> ~> Exec['systemd_reload_configuration']
		Systemd::Slice <| |>     ~> Exec['systemd_reload_configuration']
		Systemd::Snapshot <| |>  ~> Exec['systemd_reload_configuration']
		Systemd::Socket <| |>    ~> Exec['systemd_reload_configuration']
		Systemd::Swap <| |>      ~> Exec['systemd_reload_configuration']
		Systemd::Target <| |>    ~> Exec['systemd_reload_configuration']
		Systemd::Timer <| manage_service_resource != true|> ~> Exec['systemd_reload_configuration']
		Systemd::Unit <| |>      ~> Exec['systemd_reload_configuration']
	}
	
	
	# C O N F I G U R A T I O N   F I L E S
	
	if $systemd::systemd_local_gens_dir and ! $_pending_removal {
		file { 'systemd_local_gens_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/system-generators/',
			audit                         => ['content'],
			# NB: `systemctl daemon-reload` does a generator re-run, too
			notify                        => [Exec['systemd_reload_configuration']],
		}
		
		# TODO: System::Unit maybe verified by `systemd-analyze verify`
		#       and possibly fails on Require=/Wants= dependencies
		#       created by a generator
	}
	
	# system.conf
	
	if $systemd::manage_loc_systemd_conf and ! $_pending_removal {
		file { 'systemd_local_system_conf':
			ensure                        => file,
			path                          => '/etc/systemd/system.conf',
			content                       => template($systemd::systemd_system_conf_erb),
			replace                       => true,
		}
	}
	
	if $systemd::purge_loc_systemd_conf_dir and ! $_pending_removal {
		# with purge => true, and recurse => true
		# you have to manage each file in this directory individually
		file { 'systemd_local_system_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/system.conf.d/',
			purge                         => true,
			recurse                       => true,
			audit                         => ['content'],
		}
	}
	
	# journald.conf(5)
	
	if $systemd::manage_loc_journald_conf and ! $_pending_removal {
		file { 'systemd_local_journald_conf':
			ensure                        => file,
			path                          => '/etc/systemd/journald.conf',
			content                       => template($systemd::systemd_journald_conf_erb),
			replace                       => true,
		}
	}
	
	if $systemd::purge_loc_journald_conf_dir and ! $_pending_removal {
		file { 'systemd_local_journald_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/journald.conf.d/',
			purge                         => true,
			recurse                       => true,
			audit                         => ['content'],
		}
	}
	
	# logind.conf(5)
	
	if $systemd::manage_loc_logind_conf and ! $_pending_removal {
		file { 'systemd_local_logind_conf':
			ensure                        => file,
			path                          => '/etc/systemd/logind.conf',
			content                       => template($systemd::systemd_logind_conf_erb),
			replace                       => true,
		}
	}
	
	if $systemd::purge_loc_logind_conf_dir and ! $_pending_removal {
		file { 'systemd_local_logind_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/logind.conf.d/',
			purge                         => true,
			recurse                       => true,
			audit                         => ['content'],
		}
	}
	
	# networkd configuration
	
	if $systemd::purge_loc_networkd_conf_dir and ! $_pending_removal {
		file { 'systemd_local_networkd_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/network/',
			purge                         => true,
			recurse                       => true,
			audit                         => ['content'],
		}
	}
	
	# resolved.conf(5)
	
	if $systemd::manage_loc_resolved_conf and ! $_pending_removal {
		file { 'systemd_local_resolved_conf':
			ensure                        => file,
			path                          => '/etc/systemd/resolved.conf',
			content                       => template($systemd::systemd_resolved_conf_erb),
			replace                       => true,
		}
	}
	
	if $systemd::purge_loc_resolved_conf_dir and ! $_pending_removal {
		file { 'systemd_local_resolved_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/resolved.conf.d/',
			purge                         => true,
			recurse                       => true,
			audit                         => ['content'],
		}
	}
	
	if $systemd::manage_resolv_conf_symlink and ! $_pending_removal {
		file { 'systemd_resolved_symlink_to_runtime_resolv_conf':
			ensure                        => link,
			path                          => '/etc/resolv.conf',
			target                        => '/run/systemd/resolve/resolv.conf',
			replace                       => true,
		}
	}
	
	# timesyncd.conf(5)
	
	if $systemd::manage_loc_timesyncd_conf and ! $_pending_removal {
		file { 'systemd_local_timesyncd_conf':
			ensure                        => file,
			path                          => '/etc/systemd/timesyncd.conf',
			content                       => template($systemd::systemd_timesyncd_conf_erb),
			replace                       => true,
		}
	}
	
	if $systemd::purge_loc_timesyncd_conf_dir and ! $_pending_removal {
		file { 'systemd_local_timesyncd_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/timesyncd.conf.d/',
			purge                         => true,
			recurse                       => true,
			# audit content, so file removal triggers a refresh on Service['timesyncd']
			audit                         => ['content'],
		}
	}
	
	# system-preset/*.preset
	
	if $systemd::manage_loc_system_preset_dir and ! $_pending_removal {
		file { 'systemd_local_system_preset_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/system-preset/',
		}
		
		if $systemd::purge_loc_system_preset_dir {
			File['systemd_local_system_preset_dir'] {
				purge                         => true,
				recurse                       => true,
			}
		}
	}
	
	# bootchart.conf
	
	if $systemd::manage_loc_bootchart_conf and ! $_pending_removal {
		file { 'systemd_local_bootchart_conf':
			ensure                        => file,
			path                          => '/etc/systemd/bootchart.conf',
			content                       => template($systemd::systemd_bootchart_conf_erb),
			replace                       => true,
		}
	}
	
	if $systemd::purge_loc_bootchart_conf_dir and ! $_pending_removal {
		file { 'systemd_local_bootchart_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/bootchart.conf.d/',
			purge                         => true,
			recurse                       => true,
		}
	}
	
	# sleep.conf
	
	if $systemd::manage_loc_sleep_conf and ! $_pending_removal {
		file { 'systemd_local_sleep_conf':
			ensure                        => file,
			path                          => '/etc/systemd/sleep.conf',
			content                       => template($systemd::systemd_sleep_conf_erb),
			replace                       => true,
		}
	}
	
	if $systemd::purge_loc_sleep_conf_dir and ! $_pending_removal {
		file { 'systemd_local_sleep_conf_dir':
			ensure                        => directory,
			path                          => '/etc/systemd/sleep.conf.d/',
			purge                         => true,
			recurse                       => true,
		}
	}
	
	# modules-load.d(5)
	
	if $systemd::manage_modules_load_dir and ! $_pending_removal {
		file { 'systemd_modules_load_dir':
			ensure                        => directory,
			path                          => '/etc/modules-load.d/',
			purge                         => true,
			recurse                       => true,
		}
	}
	
	# machine-info(5)
	
	if $systemd::manage_machine_info and ! $_pending_removal {
		# machine-info as file
		
		if ($systemd::machine_info_provider == 'template') {
			file { 'systemd_machine_info':
				ensure                        => file,
				path                          => '/etc/machine-info',
				content                       => template('systemd/machine_info.erb'),
				replace                       => true,
			}
		}
		
		# modify machine-info with hostnamectl(1)
		# "There really should be a process running
		#  which exposes the content of a file."
		
		if ($systemd::machine_info_provider == 'hostnamectl') {
			# these two commands are used
			# along all parameters we can manipulate with hostnamectl(1)
			$_hostnamectl_bin_path         = '/usr/bin/hostnamectl'
			$_machine_info_check_shell     = '/bin/sh -c'
			
			# machine_info_pretty_hostname
			if ($systemd::machine_info_pretty_hostname != undef) {
				$_pretty_hostname              = ($systemd::machine_info_pretty_hostname == default) ? {
					true                          => '\'\'',
					default                       => $systemd::machine_info_pretty_hostname,
				}
				$_cmd_pretty_hostname          = @("EOT"/L)
						${_hostnamectl_bin_path} --no-ask-password \
						--pretty set-hostname ${_pretty_hostname}
						|-EOT
				$_cond_pretty_hostname         = @("EOT"/L$)
						${_machine_info_check_shell} \
						'. /etc/machine-info 2> /dev/null \
						[ "\${PRETTY_HOSTNAME}" = ${_pretty_hostname} ]'
						|-EOT
				
				exec { 'systemd_hostnamectl_set_pretty_hostname':
					command                       => $_cmd_pretty_hostname,
					unless                        => $_cond_pretty_hostname,
					timeout                       => 8,
				}
			}
			
			# machine_info_icon_name
			if ($systemd::machine_info_icon_name != undef) {
				$_icon_name                    = ($systemd::machine_info_icon_name == default) ? {
					true                          => '\'\'',
					default                       => $systemd::machine_info_icon_name,
				}
				$_cmd_icon_name                = @("EOT"/L)
						${_hostnamectl_bin_path} --no-ask-password \
						set-icon-name ${_icon_name}
						|-EOT
				$_cond_icon_name               = @("EOT"/L$)
						${_machine_info_check_shell} \
						'. /etc/machine-info 2> /dev/null && \
						[ "\${ICON_NAME}" = ${_icon_name} ]'
						|-EOT
				
				exec { 'systemd_hostnamectl_set_icon_name':
					command                       => $_cmd_icon_name,
					unless                        => $_cond_icon_name,
					timeout                       => 8,
				}
			}
			
			# machine_info_deployment
			if ($systemd::machine_info_deployment != undef) {
				$_deployment                   = ($systemd::machine_info_deployment == default) ? {
					true                          => '\'\'',
					default                       => $systemd::machine_info_deployment,
				}
				$_cmd_deployment               = @("EOT"/L)
						${_hostnamectl_bin_path} --no-ask-password \
						set-icon-name ${_deployment}
						|-EOT,
				$_cond_deployment              = @("EOT"/L$)
						${_machine_info_check_shell} \
						'. /etc/machine-info 2> /dev/null && \
						[ "\${DEPLOYMENT}" = ${_deployment} ]'
						|-EOT
				
				exec { 'systemd_hostnamectl_set_deployment':
					command                       => $_cmd_deployment,
					unless                        => $_cond_deployment,
					timeout                       => 8,
				}
			}
			
			# machine_info_location
			if ($systemd::machine_info_location != undef) {
				$_location                     = ($systemd::machine_info_location == default) ? {
					true                          => '\'\'',
					default                       => $systemd::machine_info_location,
				}
				$_cmd_location                = @("EOT"/L)
						${_hostnamectl_bin_path} --no-ask-password \
						set-location ${_location}
						|-EOT
				$_cond_location               = @("EOT"/L$)
						${_machine_info_check_shell} \
						'. /etc/machine-info 2> /dev/null && \
						[ "\${LOCATION}" = ${_location} ]'
						|-EOT
				
				exec { 'systemd_hostnamectl_set_location':
					command                       => $_cmd_location,
					unless                        => $_cond_location,
					timeout                       => 8,
				}
			}
		# end of provider == hostnamectl
		}
	# end of manage_machine_info
	}
	
	# sysusers.d(5)
	
	if $systemd::manage_sysusers_dir and ! $_pending_removal {
		file { 'systemd_sysusers_dir':
			ensure                        => directory,
			path                          => '/etc/sysusers.d/',
			purge                         => true,
			recurse                       => true,
		}
	}
	
	# interfaces(5)
	
	if $systemd::etc_network_interfaces_note and ! $_pending_removal {
		file { 'etc_network_interfaces':
			ensure                        => file,
			path                          => '/etc/network/interfaces',
			content                       => template($systemd::etc_network_interfaces_erb),
			replace                       => true,
		}
	}
	
	# sysctl
	
	if $systemd::manage_sysctl_conf and ! $_pending_removal {
		if ($systemd::sysctl_conf_template == undef) {
			fail('$systemd::sysctl_conf_template has to be specified')
		}
		
		file { 'sysctl_conf':
			ensure                        => file,
			path                          => '/etc/sysctl.conf',
			content                       => template($systemd::sysctl_conf_template),
			replace                       => true,
		}
	}
	
	if $systemd::manage_sysctl_directory and ! $_pending_removal {
		file { 'sysctl_conf_dir':
			ensure                        => directory,
			path                          => '/etc/sysctl.d/',
			purge                         => true,
			recurse                       => true,
		}
	}
	
	# udev.conf(5)
	
	if $systemd::manage_udev_conf and ! $_pending_removal {
		file { 'udev_conf':
			ensure                        => file,
			path                          => '/etc/udev/udev.conf',
			content                       => template($systemd::udev_conf_template),
			replace                       => true,
		}
	}
	
	# locale.conf(5)
	
	if $systemd::manage_locale_conf and ! $_pending_removal {
		# modify locale.conf with templated file
		
		if ($systemd::locale_conf_provider == 'template') {
			file { 'locale_conf':
				ensure                        => file,
				path                          => '/etc/locale.conf',
				content                       => template('systemd/locale_conf.erb'),
				replace                       => true,
			}
		}
		
		# manipulate locale.conf with localectl(1)
		
		if ($systemd::locale_conf_provider == 'localectl') {
			$_localectl_bin_path           = '/usr/bin/localectl'
			
			# TODO: use shellquote
			
			$_lang                         = ($systemd::locale_conf_lang == undef) ? {
				true                          => '',
				default                       => " 'LANG=${systemd::locale_conf_lang}'",
			}
			$_language                     = ($systemd::locale_conf_language == undef) ? {
				true                          => '',
				default                       => " 'LANGUAGE=${systemd::locale_conf_language}'",
			}
			$_lc_ctype                     = ($systemd::locale_conf_lc_ctype == undef) ? {
				true                          => '',
				default                       => " 'LC_CTYPE=${systemd::locale_conf_lc_ctype}'",
			}
			$_lc_numeric                   = ($systemd::locale_conf_lc_numeric == undef) ? {
				true                          => '',
				default                       => " 'LC_NUMERIC=${systemd::locale_conf_lc_numeric}'",
			}
			$_lc_time                      = ($systemd::locale_conf_lc_time == undef) ? {
				true                          => '',
				default                       => " 'LC_TIME=${systemd::locale_conf_lc_time}'",
			}
			$_lc_collate                   = ($systemd::locale_conf_lc_collate == undef) ? {
				true                          => '',
				default                       => " 'LC_COLLATE=${systemd::locale_conf_lc_collate}'",
			}
			$_lc_monetary                  = ($systemd::locale_conf_lc_monetary == undef) ? {
				true                          => '',
				default                       => " 'LC_MONETARY=${systemd::locale_conf_lc_monetary}'",
			}
			$_lc_messages                  = ($systemd::locale_conf_lc_messages == undef) ? {
				true                          => '',
				default                       => " 'LC_MESSAGES=${systemd::locale_conf_lc_messages}'",
			}
			$_lc_paper                     = ($systemd::locale_conf_lc_paper == undef) ? {
				true                          => '',
				default                       => " 'LC_PAPER=${systemd::locale_conf_lc_paper}'",
			}
			$_lc_name                      = ($systemd::locale_conf_lc_name == undef) ? {
				true                          => '',
				default                       => " 'LC_NAME=${systemd::locale_conf_lc_name}'",
			}
			$_lc_address                   = ($systemd::locale_conf_lc_address == undef) ? {
				true                          => '',
				default                       => " 'LC_ADDRESS=${systemd::locale_conf_lc_address}'",
			}
			$_lc_telephone                 = ($systemd::locale_conf_lc_telephone == undef) ? {
				true                          => '',
				default                       => " 'LC_TELEPHONE=${systemd::locale_conf_lc_telephone}'",
			}
			$_lc_measurement               = ($systemd::locale_conf_lc_measurement == undef) ? {
				true                          => '',
				default                       => " 'LC_MEASUREMENT=${systemd::locale_conf_lc_measurement}'",
			}
			$_lc_identification            = ($systemd::locale_conf_lc_identification == undef) ? {
				true                          => '',
				default                       => " 'LC_IDENTIFICATION=${systemd::locale_conf_lc_identification}'",
			}
			
			
			# If no locales are about to be set,
			# localectl(1) fails with "Too few arguments."
			# That's OK. The Exec resource should fail then, too.
			$_cmd_localectl_set_locale    = @("EOT"/L)
					${_localectl_bin_path} --no-ask-password --no-pager \
					set-locale\
					$_lang\
					$_language\
					$_lc_ctype\
					$_lc_numeric\
					$_lc_time\
					$_lc_collate\
					$_lc_monetary\
					$_lc_messages\
					$_lc_paper\
					$_lc_name\
					$_lc_address\
					$_lc_telephone\
					$_lc_measurement\
					$_lc_identification
					|-EOT
			# the first slash is to escape the dollar sign in puppet heredoc
			# the preceding double slashes are, to print a slash, which then
			# escapes variable interpolation in the heredoc ended by EOF
			$_cond_localectl_set_locale   = @("EOT"/L$)
					/usr/bin/env -i /bin/sh -s -o allexport << EOF
					is=`/bin/mktemp`;
					should=`/bin/mktemp`;
					. /etc/locale.conf 2> /dev/null;
					# previous command variable is noop
					:;
					/usr/bin/printenv | /usr/bin/sort > \\\${is};
					[ "${systemd::locale_conf_lang}" ] && \
					LANG=${systemd::locale_conf_lang} ;
					[ "${systemd::locale_conf_language}" ] && \
					LANGUAGE=${systemd::locale_conf_language} ;
					[ "${systemd::locale_conf_lc_ctype}" ] && \
					LC_CTYPE=${systemd::locale_conf_lc_ctype} ;
					[ "${systemd::locale_conf_lc_numeric}" ] && \
					LC_NUMERIC=${systemd::locale_conf_lc_numeric} ;
					[ "${systemd::locale_conf_lc_time}" ] && \
					LC_TIME=${systemd::locale_conf_lc_time} ;
					[ "${systemd::locale_conf_lc_collate}" ] && \
					LC_COLLATE=${systemd::locale_conf_lc_collate} ;
					[ "${systemd::locale_conf_lc_monetary}" ] && \
					LC_MONETARY=${systemd::locale_conf_lc_monetary} ;
					[ "${systemd::locale_conf_lc_messages}" ] && \
					LC_MESSAGES=${systemd::locale_conf_lc_messages} ;
					[ "${systemd::locale_conf_lc_paper}" ] && \
					LC_PAPER=${systemd::locale_conf_lc_paper} ;
					[ "${systemd::locale_conf_lc_name}" ] && \
					LC_NAME=${systemd::locale_conf_lc_name} ;
					[ "${systemd::locale_conf_lc_address}" ] && \
					LC_ADDRESS=${systemd::locale_conf_lc_address} ;
					[ "${systemd::locale_conf_lc_telephone}" ] && \
					LC_TELEPHONE=${systemd::locale_conf_lc_telephone} ;
					[ "${systemd::locale_conf_lc_measurement}" ] && \
					LC_MEASUREMENT=${systemd::locale_conf_lc_measurement} ;
					[ "${systemd::locale_conf_lc_identification}" ] && \
					LC_IDENTIFICATION=${systemd::locale_conf_lc_identification} ;
					# previous command variable becomes noop
					:;
					/usr/bin/printenv | /usr/bin/sort > \\\${should};
					/usr/bin/diff \\\${is} \\\${should};
					ES=\\\${?}
					/bin/rm \\\${is} \\\${should}
					exit \\\${ES}
					EOF
					|EOT
			
			exec { 'systemd_localectl_set_locale':
				command                       => $_cmd_localectl_set_locale,
				unless                        => $_cond_localectl_set_locale,
				timeout                       => 16,
			}
		# end of provider = 'localectl'
		}
	# end of manage_locale_conf block
	}
	
	# localtime(5)
	
	if $systemd::manage_localtime and ! $_pending_removal {
		# manage /etc/localtime with a file resource
		
		if ($systemd::localtime_provider == 'file') {
			file { 'localtime':
				ensure                        => link,
				path                          => '/etc/localtime',
				target                        => "../usr/share/zoneinfo/${systemd::localtime}",
				replace                       => true,
			}
		}
		
		# manage /etc/localtime with timedatectl(5)
		
		if ($systemd::localtime_provider == 'timedatectl') {
			$_timedatectl_bin_path        = '/usr/bin/timedatectl'
			
			$_cmd_timedatectl_set_tz      = @("EOT"/L)
					${_timedatectl_bin_path} --no-ask-password --no-pager \
					set-timezone ${systemd::localtime}
					|-EOT
			$_cond_timedatectl_set_tz     = @("EOT"/L$)
					/bin/sh -s << EOF
					is=`/bin/readlink -n /etc/localtime`;
					should=../usr/share/zoneinfo/${systemd::localtime};
					[ "\\\${is}" = "\\\${should}" ]
					EOF
					|EOT
			
			exec { 'systemd_timedatectl_set_timezone':
				command                       => $_cmd_timedatectl_set_tz,
				unless                        => $_cond_timedatectl_set_tz,
				timeout                       => 8,
			}
		# end of provider = 'timedatectl'
		}
	}
# end of class
}
# EOF
