# name: systemd
# desc: service manifest for systemd module
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# user documentation as YARD documentation {{{
# 
# This is a private class.
# 
# * It manages the `systemd-*` services (and some other).
# * It takes care of of a `systemctl daemon-reload` (if opted in).
# 
# }}}
class systemd::daemon {
	# C O N F I G U R A T I O N
	
	exec { 'systemd_reload_configuration':
		command                       => '/bin/systemctl daemon-reload',
		refreshonly                   => true,
	}
	
	# if we reload, then do it before services are (re)started
	# so they use fresh systemd service unit configuration files
	Exec['systemd_reload_configuration'] -> Service <| |>
	
	# reload manager configuration as well
	if $systemd::manage_loc_systemd_conf and ! $systemd::config::_pending_removal {
		File['systemd_local_system_conf'] ~> Exec['systemd_reload_configuration']
	}
	if $systemd::purge_loc_systemd_conf_dir and ! $systemd::config::_pending_removal {
		File['systemd_local_system_conf_dir'] ~> Exec['systemd_reload_configuration']
	}
	
	
	# S E R V I C E S # # # # # # # # # # # # # # # # # # # # # # # # # # # #
	
	# systemd-networkd(8) ---------------------------------------------------
	
	if ($systemd::systemd_networkd_ensure != undef) {
		service { 'networkd':
			name                          => 'systemd-networkd',
			ensure                        => $systemd::systemd_network_ensure,
			enable                        => $systemd::systemd_network_enable,
		}
		
		if $systemd::purge_loc_networkd_conf_dir and ! $_pending_removal {
			File['systemd_local_networkd_conf_dir'] ~> Service['networkd']
		}
	}
	
	# systemd-resolved(8) ---------------------------------------------------
	
	if ($systemd::systemd_resolved_ensure != undef) {
		service { 'resolved':
			name                          => 'systemd-resolved',
			ensure                        => $systemd::systemd_resolved_ensure,
			enable                        => $systemd::systemd_resolved_enable,
		}
		
		# subscribe to systemd-resolved(8) configuration file
		# if they're managed by this module
		if $systemd::manage_loc_resolved_conf and ! $systemd::config::_pending_removal {
			File['systemd_local_resolved_conf'] ~> Service['resolved']
		}
		if $systemd::purge_loc_resolved_conf_dir and ! $systemd::config::_pending_removal {
			File['systemd_local_resolved_conf_dir'] ~> Service['resolved']
		}
	}
	
	# systemd-timesyncd(8) --------------------------------------------------
	
	if ($systemd::systemd_timesyncd_ensure != undef) {
		service { 'timesyncd':
			name                          => 'systemd-timesyncd',
			ensure                        => $systemd::systemd_timesyncd_ensure,
			enable                        => $systemd::systemd_timesyncd_enable,
		}
		
		if $systemd::manage_loc_timesyncd_conf and ! $systemd::config::_pending_removal {
			File['systemd_local_timesyncd_conf'] ~> Service['timesyncd']
		}
		if $systemd::purge_loc_timesyncd_conf_dir and ! $systemd::config::_pending_removal {
			File['systemd_local_timesyncd_conf_dir'] ~> Service['timesyncd']
		}
	}
}
# EOF
