# name: system control able
# desc: enable/disable wrapper
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# user documentation as YARD documentation {{{
# }}}
define systemd::systemctl::able (
	# choose between systemd::systemctl::enable and systemd::systemctl::disable
	Optional[Boolean]        $able                          = undef,
	# input validation
	Boolean                  $reject_undef_able             = true,
	Boolean                  $reject_undef_able_unless_noop = false,
	# preparations
	# systemctl enable/disable call per se:
	Optional[Enum['system', 'user', 'runtime', 'global']]
	                         $scope                         = undef,
	Variant[Boolean, Tuple[Boolean, 2, 2], Undef]
	                         $force                         = undef,
	Variant[Boolean, Tuple[Boolean, 2, 2], Undef]
	                         $now                           = undef,
	Optional[Tuple[Enum['flag', 'stop'], Enum['flag', 'start'], 2, 2]]
	                         $now_implementation            = undef,
	Variant[Boolean, Tuple[Boolean, 2, 2], Undef]
	                         $reload                        = undef,
	Optional[Struct[{
			username                      => Optional[Pattern[/^\w+$/]],
			hostname                      => Pattern[/^\w+$/],
			container_name                => Optional[Pattern[/^\w+$/]],
		}]] $host = undef,
	Optional[Pattern[/^\w+$/]]
	                         $machine                       = undef,
	Optional[Pattern[/^\/$/, /^\/\S\/$/]]
	                         $root                          = undef,
	# systemctl enable/disable call : puppet exec resource tweaks
	Optional[Pattern[/^\S+$/]]
	                         $user                          = undef,
	Variant[Enum['on_failure'], Boolean, Undef]
	                         $logoutput                     = undef,
	Optional[Boolean]        $refreshonly                   = undef,
	# systemctl enable/disable call aftermath
	Variant[Boolean, Tuple[Boolean, 2, 2], Undef]
	                         $trigger_daemon_reload         = undef,
) {
	#                                                                        #
	# I N P U T   V A L I D A T I O N                                        #
	#                                                                        #
	
	# Because $able is mainly fed via a variable (if we knew in advance
	# to either enable or disable a unit, we'd directly write so),
	# it is realistic that the variable value is not defined.
	# Here, with $reject_undef_able* we take this into account.
	# However, by default we fail(), because an undefined value could
	# simply result from misspelling a variable name.
	
	# 2017-03-12 KB: $able will not be extended by a fourth allowed value
	#                meaning "it is alright, do nothing"
	
	# fail with non-satisfying $able value (unless turned off)
	if ($able == undef and $reject_undef_able == true) {
		# disable sabotage resource creation in noop-mode
		if (($noop == true and $reject_undef_able_unless_noop) or ($noop == false)) {
			fail(sprintf('%s[%s]: $able is not defined', $module_name, $title))
		}
	}
	
	
	#                                                                        #
	# M A P P I N G                                                          #
	#                                                                        #
	
	# Usually $able is specified via a variable
	# (otherwise we would directly create an enable/disable resource).
	# Because of that, we'd like to specify some settings
	# depending on whether we're disabling or enabling a unit.
	# To achieve this, some parameters -- where it is reasonable to
	# distinguish between $able values -- can be specified as 2-tuples.
	# 
	# However, to keep the user's code clean, they can abbreviate and write
	# one element where for both cases the same behavior is acceptable.
	# But in the end resource creation happens with normalized values
	# (see below), so we have to encapsulate single values to tuples here.
	
	$_force                        = ($force !~ Tuple[Boolean, 2, 2]) ? {
		true                          => [$force, $force],
		default                       => $force,
	}
	
	$_now                          = ($now !~ Tuple[Boolean, 2, 2]) ? {
		true                          => [$now, $now],
		default                       => $now,
	}
	
	$_now_implementation           = ($now_implementation !~ Tuple[String, 2, 2]) ? {
		true                          => [$now_implementation, $now_implementation],
		default                       => $now_implementation,
	}
	
	$_reload                       = ($reload !~ Tuple[Boolean, 2, 2]) ? {
		true                          => [$reload, $reload],
		default                       => $reload,
	}
	
	$_trigger_daemon_reload        = ($trigger_daemon_reload !~ Tuple[Boolean, 2, 2]) ? {
		true                          => [$trigger_daemon_reload, $trigger_daemon_reload],
		default                       => $trigger_daemon_reload,
	}
	
	
	#                                                                        #
	# R E S O U R C E S                                                      #
	#                                                                        #
	
	# do an equal-comparison because $reject_undef_able might be false
	if ($able == true) {
		systemd::systemctl::enable { $title:
			scope                         => $scope,
			force                         => $_force[1],
			now                           => $_now[1],
			now_implementation            => $_now_implementation[1],
			reload                        => $_reload[1],
			host                          => $host,
			machine                       => $machine,
			root                          => $root,
			user                          => $user,
			logoutput                     => $logoutput,
			refreshonly                   => $refreshonly,
			trigger_daemon_reload         => $_trigger_daemon_reload[1],
		}
	}
	
	# do an equal-comparison because $reject_undef_able might be false
	if ($able == false) {
		systemd::systemctl::disable { $title:
			scope                         => $scope,
			force                         => $_force[0],
			now                           => $_now[0],
			now_implementation            => $_now_implementation[0],
			reload                        => $_reload[0],
			host                          => $host,
			machine                       => $machine,
			root                          => $root,
			user                          => $user,
			logoutput                     => $logoutput,
			refreshonly                   => $refreshonly,
			trigger_daemon_reload         => $_trigger_daemon_reload[0],
		}
	}
}
# EOF
