# name: system control disable
# desc: defined type for calling systemctl
# auth: Kai Burghardt <wiz at KaiBurghardt.de>
#  vim: set tw=140 ts=4 noet bs=eol,start,indent fdm=marker fdl=0:

# user documentation as YARD documentation {{{
# }}}
define systemd::systemctl::disable (
	# condition check
	Boolean                  $masked_is_disabled            = true,
	Boolean                  $runtime_masked_is_disabled    = true,
	Boolean                  $static_is_disabled            = true,
	Boolean                  $indirect_is_disabled          = true,
	# systemctl disable call
	Optional[Enum['system', 'user', 'runtime', 'global']]
	                         $scope                         = undef,
	Boolean                  $force                         = false,
	Boolean                  $now                           = false,
	Enum['flag', 'stop']     $now_implementation            = 'flag',
	Boolean                  $reload                        = true,
	Optional[Struct[{
			username                      => Optional[Pattern[/^\w+$/]],
			hostname                      => Pattern[/^\w+$/],
			container_name                => Optional[Pattern[/^\w+$/]],
		}]] $host = undef,
	Optional[Pattern[/^\w+$/]]
	                         $machine                       = undef,
	Optional[Pattern[/^\/$/, /^\/\S\/$/]]
	                         $root                          = undef,
	# systemctl disable call : puppet exec resource tweaks
	Optional[Pattern[/^\S+$/]]
	                         $user                          = undef,
	Variant[Enum['on_failure'], Boolean, Undef]
	                         $logoutput                     = undef,
	Optional[Boolean]        $refreshonly                   = undef,
	# systemctl disable call aftermath
	Boolean                  $trigger_daemon_reload         = false,
) {
	#                                                                        #
	# I N P U T   V A L I D A T I O N                                        #
	#                                                                        #
	
	if ($title =~ / /) {
		fail(sprintf('%s[%s]: title may not contain spaces', $module_name, $title))
	}
	
	# "This command expects valid unit names only,
	# it does not accept paths to unit files." -- systemctl(1)
	if ($title =~ /\//) {
		fail(sprintf('%s[%s]: title cannot specify a path to a unit file', $module_name, $title))
	}
	
	
	#                                                                        #
	# V A R I A B L E   E V A L U A T I O N                                  #
	#                                                                        #
	
	# map title, because assuming you declare two resources of this type,
	# one its title reads 'ssh' and the other one 'ssh.service',
	# we do not want to act twice on the same unit,
	# but provoke an error later when declaring the Exec[$_title] resource
	$_unit_name                    = regsubst($title, /\..*$/, '')
	$_unit_suffix                  = regsubst(regsubst($title, /^.*\./, ''), $title, '')
	$_unit_type                    = $_unit_suffix ? {
		''                            => 'service',
		default                       => $_unit_suffix,
	}
	## exec resource title
	$_title                        = "systemctl_disable_${_unit_name}_${_unit_type}"
	
	## command basis
	$_c                            = '/bin/systemctl'
	$_np                           = '--no-pager'
	
	# --system --user --runtime --global
	$_scope                        = ($scope != undef) ? {
		true                          => "--${scope}",
		default                       => '',
	}
	
	# --force: overwrite any existing conflicting symlinks
	$_force                        = $force ? {
		true                          => '--force',
		default                       => '',
	}
	
	# --now: unit also gets stopped
	$_now                          = $now ? {
		true                          => $now_implementation ? {
				'flag'                        => '--now',
				default                       => '',
			},
		default                       => '',
	}
	
	# --no-reload: no implict daemon configuration reload after enable
	$_reload                       = $reload ? {
		false                         => '--no-reload',
		default                       => '',
	}
	
	# --host=: execute the operation remotely
	if ($host != undef) {
		$_host_user                    = ($host['username'] != undef) ? {
			false                         => "${host[username]}@",
			default                       => '',
		}
		$_host_container               = ($host['container_name'] != undef) ? {
			false                         => ":${host[container_name]}",
			default                       => '',
		}
		$_host                         = "--host=${_host_user}${host[hostname]}${_host_container}"
	} else {
		$_host                         = ''
	}
	
	# --machine=: execute operation on a local container
	$_machine                      = ($machine != undef) ? {
		true                          => "--machine=${machine}",
		default                       => '',
	}
	
	# --root=: alternate root path when looking for unit files
	$_root                         = ($root != undef) ? {
		true                          => "--root=${root}",
		default                       => '',
	}
	
	$_o                            = [$_np, $_scope, $_force, $_reload, $_now, $_host, $_machine, $_root]
	$_opts                         = $_o.filter |$x| { $x =~ /./ }
	# inline_template for backward compatibility PP 3.8 with future parser enabled
	$_options                      = inline_template(" <%= @_opts.join(' ') %>")
	
	# exec checks
	$_masked                       = $masked_is_disabled ? {
		true                          => 'masked',
		default                       => '',
	}
	$_masked_runtime               = $runtime_masked_is_disabled ? {
		true                          => 'masked-runtime',
		default                       => '',
	}
	$_static                       = $static_is_disabled ? {
		true                          => 'static',
		default                       => '',
	}
	$_indirect                     = $indirect_is_enabled ? {
		true                          => 'indirect',
		default                       => '',
	}
	
	$_d                            = ['disabled', $_masked, $_masked_runtime, $_static, $_indirect]
	$_dis                          = $_d.filter |$x| { $x =~ /./ }
	# inline_template() instead reduce() for backwards compatibility w/ PP 3.8
	$_disabled                     = inline_template("<%= @_dis.join('|') %>")
	$_unless                       = "${_c} ${_np} is-enabled ${title} | /bin/grep -xEq '(${_disabled})'"
	
	
	#                                                                        #
	# R E S O U R C E   D E C L A R A T I O N S                              #
	#                                                                        #
	
	exec { $_title:
		user                          => $user,
		refreshonly                   => $refreshonly,
		unless                        => $_unless,
		logoutput                     => $logoutput,
		command                       => "${_c} ${_options} disable ${title}",
		tries                         => 1,
	}
	
	#if ($now and $now_implementation == 'stop') {
	#	systemctl::stop { $title:
	#		user                          => $user,
	#		refreshonly                   => $refreshonly,
	#		logoutput                     => $logoutput,
	#		require                       => Exec[$_title],
	#	}
	#}
	
	if $trigger_daemon_reload {
		Exec[$_title] ~> Exec['systemd_reload_configuration']
	}
}
# EOF
